@@ -27,11 +27,11 @@
 
 from __future__ import absolute_import
 
-import os
 import six
 import struct
 import re
 import logging
+import random
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.backends import default_backend
 from cryptography import x509
@@ -94,11 +94,11 @@ class APPLICATION(BitflagEnum):
         return APPLICATION.OPGP | APPLICATION.OATH | APPLICATION.PIV
 
     def __str__(self):
-        if self == self.U2F:
+        if self == APPLICATION.U2F:
             return 'FIDO U2F'
-        elif self == self.FIDO2:
+        elif self == APPLICATION.FIDO2:
             return 'FIDO2'
-        elif self == self.OPGP:
+        elif self == APPLICATION.OPGP:
             return 'OpenPGP'
         else:
             return self.name
@@ -111,17 +111,20 @@ class FORM_FACTOR(IntEnum):
     USB_A_NANO = 0x02
     USB_C_KEYCHAIN = 0x03
     USB_C_NANO = 0x04
+    USB_C_LIGHTNING = 0x05
 
     def __str__(self):
-        if self == self.USB_A_KEYCHAIN:
+        if self == FORM_FACTOR.USB_A_KEYCHAIN:
             return 'Keychain (USB-A)'
-        elif self == self.USB_A_NANO:
+        elif self == FORM_FACTOR.USB_A_NANO:
             return 'Nano (USB-A)'
-        elif self == self.USB_C_KEYCHAIN:
+        elif self == FORM_FACTOR.USB_C_KEYCHAIN:
             return 'Keychain (USB-C)'
-        elif self == self.USB_C_NANO:
+        elif self == FORM_FACTOR.USB_C_NANO:
             return 'Nano (USB-C)'
-        elif self == self.UNKNOWN:
+        elif self == FORM_FACTOR.USB_C_LIGHTNING:
+            return 'Keychain (USB-C, Lightning)'
+        elif self == FORM_FACTOR.UNKNOWN:
             return 'Unknown.'
 
     @classmethod
@@ -231,20 +234,36 @@ class Mode(object):
     __hash__ = None
 
 
+def _tlv_parse_tag(data, offs=0):
+    t = six.indexbytes(data, offs)
+    if t & 0x1f != 0x1f:
+        return t, 1
+    else:
+        t = t << 8 | six.indexbytes(data, offs+1)
+        return t, 2
+
+
+def _tlv_parse_length(data, offs=0):
+    ln = six.indexbytes(data, offs)
+    offs += 1
+    if ln > 0x80:
+        n_bytes = ln - 0x80
+        ln = bytes2int(data[offs:offs + n_bytes])
+    else:
+        n_bytes = 0
+    return ln, n_bytes + 1
+
+
 class Tlv(bytes):
 
     @property
     def tag(self):
-        return six.indexbytes(self, 0)
+        return _tlv_parse_tag(self)[0]
 
     @property
     def length(self):
-        ln = six.indexbytes(self, 1)
-        offs = 2
-        if ln > 0x80:
-            n_bytes = ln - 0x80
-            ln = bytes2int(self[offs:offs + n_bytes])
-        return ln
+        _, offs = _tlv_parse_tag(self)
+        return _tlv_parse_length(self, offs)[0]
 
     @property
     def value(self):
@@ -267,13 +286,9 @@ class Tlv(bytes):
                 tag = data
                 value = b''
             else:  # Called with binary TLV data
-                tag = six.indexbytes(data, 0)
-                ln = six.indexbytes(data, 1)
-                offs = 2
-                if ln > 0x80:
-                    n_bytes = ln - 0x80
-                    ln = bytes2int(data[offs:offs + n_bytes])
-                    offs = offs + n_bytes
+                tag, tag_ln = _tlv_parse_tag(data)
+                ln, ln_ln = _tlv_parse_length(data, tag_ln)
+                offs = tag_ln + ln_ln
                 value = data[offs:offs+ln]
         elif len(args) == 2:  # Called with tag and value.
             (tag, value) = args
@@ -281,7 +296,15 @@ class Tlv(bytes):
             raise TypeError('{}() takes at most 2 arguments ({} given)'.format(
                 cls, len(args)))
 
-        data = bytearray([tag])
+        data = bytearray([])
+        if tag <= 0xff:
+            data.append(tag)
+        else:
+            tag_1 = tag >> 8
+            if tag_1 > 0xff or tag_1 & 0x1f != 0x1f:
+                raise ValueError('Unsupported tag value')
+            tag_2 = tag & 0xff
+            data.extend([tag_1, tag_2])
         length = len(value)
         if length < 0x80:
             data.append(length)
@@ -293,6 +316,36 @@ class Tlv(bytes):
 
         return super(Tlv, cls).__new__(cls, bytes(data))
 
+    @classmethod
+    def parse_from(cls, data):
+        tlv = cls(data)
+        return tlv, data[len(tlv):]
+
+    @classmethod
+    def parse_list(cls, data):
+        res = []
+        while data:
+            tlv, data = cls.parse_from(data)
+            res.append(tlv)
+        return res
+
+    @classmethod
+    def parse_dict(cls, data):
+        return dict((tlv.tag, tlv.value) for tlv in cls.parse_list(data))
+
+    @classmethod
+    def unpack(cls, tag, data):
+        tlv = cls(data)
+        if tlv.tag != tag:
+            raise ValueError('Wrong tag, got {:02x} expected {:02x}'.format(
+                tlv.tag,
+                tag
+            ))
+        return tlv.value
+
+
+parse_tlvs = Tlv.parse_list  # Deprecated, use Tlv.parse_list directly
+
 
 class MissingLibrary(object):
     def __init__(self, message):
@@ -302,15 +355,6 @@ class MissingLibrary(object):
         raise AttributeError(self._message)
 
 
-def parse_tlvs(data):
-    res = []
-    while data:
-        tlv = Tlv(data)
-        data = data[len(tlv):]
-        res.append(tlv)
-    return res
-
-
 def int2bytes(value):
     buf = []
     while value > 0xff:
@@ -351,14 +395,12 @@ def modhex_encode(value):
 
 
 def generate_static_pw(
-    length, keyboard_layout=KEYBOARD_LAYOUT.MODHEX,
+        length,
+        keyboard_layout=KEYBOARD_LAYOUT.MODHEX,
         blacklist=DEFAULT_PW_CHAR_BLACKLIST):
-    data = os.urandom(length)
-    keys = ''.join([
-        k for k in keyboard_layout.value.keys() if k not in blacklist]).encode()
-    return bytes(
-            bytearray(six.indexbytes(
-                keys, d % len(keys)) for d in six.iterbytes(data)))
+    chars = [k for k in keyboard_layout.value.keys() if k not in blacklist]
+    sr = random.SystemRandom()
+    return ''.join([sr.choice(chars) for _ in range(length)])
 
 
 def format_code(code, digits=6, steam=False):
@@ -473,7 +515,9 @@ def parse_certificates(data, password):
                         PEM_IDENTIFIER + cert, default_backend()))
             except Exception:
                 pass
-        return certs
+        # Could be valid PEM but not certificates.
+        if len(certs) > 0:
+            return certs
 
     # PKCS12
     if is_pkcs12(data):
